﻿% -----------------------------------------------------------------------------
% File: FLAME_samples.pl
% Content: Sample uses of the FLAME framework
% Author: Amador Durán Toro
% Version: 2.1
% Date: 12/03/2015
% -----------------------------------------------------------------------------

% Core files
:- consult( 'afl/afl.pl'     ). % FLAME AFL layer
:- consult( 'cml/bfm/bfm.pl' ). % FLAME CML layer for BFM

% SPL instances: spl_db( ID, spl( Features, Model ) )

% Ill-formed SPL for testing the checking predicates

spl_db( bad_spl,
  spl(
    [ f1, f2, f3 ],
    bfm( feature( f1, [
            mandatory( feature( f2, [] ) ),
            mandatory( feature( f4, [] ) ),
            optional( feature( f2, [] ) ),
            optional( feature( f2, [
              only_one( [ feature( f1, [] ), feature( f5, [] ), feature( f4, [] ) ] )
            ] ) )
          ] ),
          [
            requires( f6, f1 ),
            excludes( f1, f1 )
          ]
    )
  )
).

% SPL used in the survey  "Automated Analysis of Feature Models 20 years Later"

spl_db( survey_spl,
  spl(
    [ mobile_phone, calls, gps, screen, basic, color, high_resolution, media, camera, mp3 ],
    bfm( feature( mobile_phone, [
            mandatory( feature( calls, [] ) ),
            optional( feature( gps, [] ) ),
            mandatory( feature( screen, [
               only_one( [ 
                  feature( basic, [] ),
                  feature( color, [] ),
                  feature( high_resolution, [] )
               ] )
            ] ) ),
            optional( feature( media, [
               one_or_more( [ feature( camera, [] ), feature( mp3, [] ) ] )
            ] ) )
          ] ),
          [
            excludes( gps, basic ),
            requires( camera, high_resolution )
          ]
    )
  )
).

% SPL for testing atomic sets.

spl_db( atomic_sets_spl,
  spl(
    [ a, b, c, d, e, f, g ],
    bfm( feature( a, [
         optional( feature( b, [
             mandatory( feature( e, [] ) )
         ] ) ),
         mandatory( feature( c, [] ) ),
         mandatory( feature( d, [
             one_or_more( [ feature( f, [] ), feature( g, [] ) ] )
         ] ) )
    ] ),
    [ requires( e, f ),
      requires( f, e )
    ]
    )
  )
).

spl_db( equiv5_a,
  spl(
    [ root, f1, f3, f4, f5, f6, f7, f8, f9, f2 ],
    bfm( feature( root, [
         mandatory( feature( f1, [
            mandatory( feature( f3, [] ) ),
            mandatory( feature( f4, [] ) ),
            mandatory( feature( f5, [] ) ),
            mandatory( feature( f6, [] ) ),
            mandatory( feature( f7, [] ) ),
            only_one( [ feature( f8, [] ), feature( f9, [] ) ] )
         ] ) ),
         mandatory( feature( f2, [] ) )
    ] ),
    [ ]
   )
 )
).

spl_db( equiv5_b,
         spl( [ root, f1, f2, f3, f4, f5, f6, f7, f8, f9 ],
                bfm( feature( root, [
                        mandatory( feature( f1, [] ) ),
                        mandatory( feature( f2, [] ) ),
                        mandatory( feature( f3, [] ) ),
                        mandatory( feature( f4, [
                                mandatory( feature( f5, [] ) ),
                                mandatory( feature( f6, [] ) ),
                                mandatory( feature( f7, [] ) ),
                                only_one( [
                                  feature( f8, [] ),
                                  feature( f9, [] )
                                ] )
                        ] ) )
                ] ),
                [ ]
                )
        )
).

spl_db( generalization,
          spl( [ a, b, c, d ],
                bfm( feature( a, [
                   optional( feature( b, [
                      optional( feature( d, [] ) )
                   ] ) ),
                   optional( feature( c, [] ) )
                ] ),
                [ ]
          )
       )
).

spl_db( sosym_spl,
  spl( [ a, b, c, d, e, f, g ],
    bfm(
      feature( a, [
        optional( feature( b, [
          one_or_more( [
            feature( d, [] ),
            feature( e, [] )
          ] )
        ] ) ),
        mandatory( feature( c, [] ) ),
        only_one( [
          feature( f, [] ),
          feature( g, [] )
        ] )
      ] ),
      [ requires( g, f ) ]
    )
  )
).

% Sample usage from Prolog prompt: analyze( survey_spl ).

analyze( SPL_ID ) :-
  spl_db( SPL_ID, SPL ),

  write( 'Checking ' ), write( SPL_ID ), nl,
  check_spl( SPL ),

  write( 'Products of ' ), write( SPL_ID ), nl,
  products_verbose( SPL, PRDS ),

  nop( SPL, NOP ),
  write( 'Number of products = ' ), write( NOP ), nl,

  core_features( SPL, CORE ),
  write( 'Core features = ' ), write( CORE ), nl,

  variant_features( SPL, VARIANT ),
  write( 'Variant features = ' ), write( VARIANT ), nl,

  dead_features( SPL, DEAD ),
  write( 'Dead features = ' ), write( DEAD ), nl,

  unique_features( SPL, UNIQUE ),
  write( 'Unique features = ' ), write( UNIQUE ), nl,

  atomic_sets( SPL, ATOMS ),
  write( 'Atomic sets: '), write( ATOMS ), nl,

  variability( SPL, V ),
  write( 'Variability = ' ), format( '~2f%', V * 100 ), nl,

  partial_variability( SPL, PV ),
  write( 'Partial variability = ' ), format( '~2f%', PV * 100 ), nl,

  homogeneity_old( SPL, H_OLD ),
  write( '(old) Homogeneity = ' ), format( '~2f%', H_OLD * 100 ), nl,

  homogeneity( SPL, H ),
  write( '(new) Homogeneity = ' ), format( '~2f%', H * 100 ), nl,

  !.

main:-
    write( '------------------------------' ), nl,
    write( ' FLAME 1.0                    ' ), nl,
    write( '------------------------------' ), nl,
    write( ' Amador Durán - 2015          ' ), nl,
    write( ' University of Seville (Spain)' ), nl,
    write( '------------------------------' ), nl,
    nl,
    write( 'Included sample SPLs:         ' ), nl,
    write( '   bad_spl                    ' ), nl,
    write( '   survey_spl                 ' ), nl,
    write( '   atomic_sets_spl            ' ), nl,
    write( '   equiv5_a                   ' ), nl,
    write( '   equiv5_b                   ' ), nl,
    write( '   generalization             ' ), nl,
    write( '   sosym_spl                  ' ), nl,
    nl,
    write( 'Type the SPL name to analyze including an ending period: ' ),
    read( SPL ),
    analyze( SPL ),
    write( 'Press a key to end the program: ' ),
    get_single_char(_),
    halt.
