﻿% -----------------------------------------------------------------------------
% File: atomic_sets.pl
% Content: Atomic sets of FLAME (refactored)
% Author: Amador Durán Toro
% Version: 2.0
% Date: 22/07/2014
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% Z definitions
%   atomicSets : SPL -> F F Feature
%
% Prolog predicates
%   atomic_sets( SPL, [ [ Feature ] ] )
%   potential_atomic_sets( SPL, [ [ Feature ] ] )
%   potential_atomic_set( [ Feature ], [ [ Feature ] ] )
%
% These predicates compute the atomic sets of an SPL.
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% atomic_sets( SPL, [ [ Feature ] ] )
% It computes the (maximal) atomic sets of an SPL
% -----------------------------------------------------------------------------

atomic_sets( spl( F, M ), ATOMS ) :-
  potential_atomic_sets( spl( F, M ), POTATOMS ),
  findall( A, ( member( A, POTATOMS ), maximal( A, POTATOMS ) ), ATOMS ).

% -----------------------------------------------------------------------------
% potential_atomic_sets( SPL, [ [ Feature ] ] )
% It computes the potential atomic sets of an SPL using the auxiliary predicate
% atomicSet.
% -----------------------------------------------------------------------------

potential_atomic_sets( spl( F, M ), POTATOMS ) :-
  power_set( F, PWSET0 ),
  select( [], PWSET0, PWSET1 ), % extract empty set
  products( spl( F, M ), PRDS ),
  findall( A0, ( member( A0, PWSET1 ), 
                 potential_atomic_set( A0, PRDS ) ), POTATOMS ).

potential_atomic_set( A0, PRDS ) :-
  forall( member( P, PRDS ), ( subset( A0, P ); disjoint( A0, P ) ) ).
