﻿% -----------------------------------------------------------------------------
% File: bags.pl
% Content: Bag toolkit for FLAME
% Author: Amador Durán Toro
% Version: 1.0
% Date: 18/03/2015
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% Prolog predicates
%   bag_domain( B, D )
%   bag_range( B, R )
%   bag_to_pairs( B, P )
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% bag_domain( B, D )
% It computes the domain of a bag, i.e. a list with duplicates.
% -----------------------------------------------------------------------------

%bag_domain( B, D ) :-
%  bag_to_pairs( B, P ),
%  pairs_keys( P, D ).
  
bag_domain( B, D ) :-
  list_to_set( B, D ).

% -----------------------------------------------------------------------------
% bag_range( B, R )
% It computes the range of a bag, i.e. a list with duplicates.
% -----------------------------------------------------------------------------

bag_range( B, R ) :-
  bag_to_pairs( B, P ),
  pairs_values( P, V ),
  list_to_set( V, R ).

% -----------------------------------------------------------------------------
% bag_to_pairs( B, P )
% P is a list of pairs (X-N), in which for each X in B, N is the number of
% times it appears in B.
% -----------------------------------------------------------------------------

bag_to_pairs( B, P ) :-
  aggregate_all( set( X-N ),
                 ( member( X, B ), aggregate( count, member( X, B ), N ) ), P ).