﻿% -----------------------------------------------------------------------------
% File: commonality.pl
% Content: Commonality indicator of FLAME (refactored)
% Author: Amador Durán Toro
% Version: 2.1
% Date: 01/03/2015
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% Z definitions
%   commonality : SPL x Configuration -> R
%   commonality_feature : SPL x Feature -> R
%
% Prolog predicates
%   commonality( SPL, CONF, C )
%   commonality_feature( SPL, F, C )
%
% It computes the commonality factor of a configuration or a single feature
% in an SPL.
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% commonality( SPL, CONF, C )
% It computes the commonality factor of a configuration in an SPL
% -----------------------------------------------------------------------------

commonality( spl( F, M ), configuration( _, _ ), 0 ) :-
  void( spl( F, M ) ), !.

commonality( spl( F, M ), configuration( S, R ), C ) :-
  filter( spl( F, M ), configuration( S, R ), FILTERED ),
  nop( spl( F, M ), NOP ),
  length( FILTERED, NOFP ),
  C is NOFP / NOP.

% -----------------------------------------------------------------------------
% commonality_feature( SPL, Fi, C )
% It computes the commonality factor of a Fi feature in an SPL
% -----------------------------------------------------------------------------

commonality_feature( spl( F, M ), Fi, C ) :-
  commonality( spl( F, M ), configuration( [Fi], [] ), C ).
