﻿% -----------------------------------------------------------------------------
% File: homogeneity.pl
% Content: Homogeneity indicator of FLAME (refactored)
% Author: Amador Durán Toro
% Version: 3.0
% Date: 01/03/2015
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% Z definitions
%   homogeneity : SPL -> R
%   homogeneity_old : SPL -> R
%
% Prolog predicates
%   homogeneity( SPL, H )
%   homogeneity_old( SPL, H )
%
% It computes the homogeneity of an SPL.
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% homogeneity( SPL, P )
% It computes the (new) homogeneity of an SPL.
% -----------------------------------------------------------------------------

homogeneity( spl( F, M ), H ) :-
  aggregate_all( sum( Ci ),
    ( member( Fi, F ), commonality_feature( spl( F, M ), Fi, Ci ) ), C ),
  length( F, NOF ),
  H is ( C / NOF ).

% -----------------------------------------------------------------------------
% homogeneity_old( SPL, P )
% It computes the (old) homogeneity of an SPL.
% -----------------------------------------------------------------------------

homogeneity_old( spl( F, M ), H ) :-
  unique_features( spl( F, M ), UF ),
  length( UF, NOUF ),
  length( F, NOF ),
  H is 1 - ( NOUF / NOF ).
