﻿% -----------------------------------------------------------------------------
% File: products.pl
% Content: Product functions and relations of FLAME
% Author: Amador Durán Toro
% Version: 3.0
% Date: 03/03/2015
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% Z definitions
%   products : SPL -> F Product
%
% Prolog predicates (public)
%   products( spl( Features, Model ), [ Products ] )
%
% It returns the set of valid products of an SPL.
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% products( spl( Features, Model ), Products )
% It dinamically computes the set of products of an SPL using the valid
% predicate.
% -----------------------------------------------------------------------------

:- dynamic products_db/2. % to assert product sets and avoid recomputation

% If second argument is bound, it is compared with database content.

products( spl( F, M ), PRDS ) :-
  products_db( spl( F, M ), PRDS_AUX ), !,
  PRDS = PRDS_AUX.

% If second argument is bound, the product set is computed and stored,
% and then, it is compared with database content.

products( spl( F, M ), PRDS ) :-
  power_set( F, PWSET0 ),
  select( [], PWSET0, PWSET1 ), % extract empty set
  findall( P, ( member( P, PWSET1 ), valid( P, spl( F, M ) ) ), PRDS_AUX ),
  assert( products_db( spl( F, M ), PRDS_AUX ) ),
  PRDS = PRDS_AUX.
