﻿% -----------------------------------------------------------------------------
% File: products_verbose.pl
% Content: Verbose product function of FLAME (refactored)
% Author: Amador Durán Toro
% Version: 2.0
% Date: 22/07/2014
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% products_verbose( spl( Features, Model ), Products )
% Same as products but showing progress. It does not use findall but a classic
% accumulator-based auxiliary predicate.
% -----------------------------------------------------------------------------

products_verbose( spl( F, M ), PRDS ) :-
  write( 'Computing products...' ), nl,
    power_set( F, PWR ),
    select( [], PWR, PWR1 ), % take the emptyset away
  length( PWR1, NOPP ),
  write( 'Checking ' ),
  writeq( NOPP ),
  write( ' potential products:' ), nl,
    products_aux( PWR1, spl( F, M ), [], PRDS ),
  length( PRDS, NOP ),
  write( 'There are ' ),
  writeq( NOP ),
  write( ' products in the SPL.' ), nl.

products_aux( [], _, PRDS, PRDS ).

products_aux( [ Pi | P ], spl( F, M ), ACC1, PRDS ) :-
  valid( Pi, spl( F, M ) ), !,
  why_not_valid( Pi, spl( F, M ) ),
  append( [ Pi ], ACC1, ACC2 ),
  products_aux( P, spl( F, M ), ACC2, PRDS ).

products_aux( [ Pi | P ], spl( F, M ), ACC, PRDS ) :-
  %why_not_valid( Pi, spl( F, M ) ),
  products_aux( P, spl( F, M ), ACC, PRDS ).
