﻿% -----------------------------------------------------------------------------
% File: valid_configuration.pl
% Content: Valid configuration relations of FLAME (refactored)
% Author: Amador Durán Toro
% Version: 2.0
% Date: 22/07/2014
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% Z definitions
%   _valid_p_c_ : Product <-> Configuration
%   _valid_c_   : Configuration <-> SPL
%
% Prolog predicates
%   valid_p_c( Product, Configuration )
%   valid_c( Configuration, spl( Features, Model ) )
%
% The first predicate checks if a product is valid for a configuration.
% The second checks if a configuration is valid for an SPL.
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% valid_p_c( Product, Configuration( Selected, Removed ) )
% It checks if a product is valid for a configuration.
% -----------------------------------------------------------------------------

valid_p_c( P, configuration( S, R ) ) :-
  subset( S, P ),
  intersection( R, P, [] ).

% -----------------------------------------------------------------------------
% valid( Configuration( Selected, Removed ), spl( Features, Model ) )
% It checks if a configuration is valid for an SPL.
% -----------------------------------------------------------------------------

valid_c( configuration( S, R ), spl( F, M ) ) :-
  union( S, R, U ),
  subset( U, F ),
  products( spl( F, M ), PRDS ),
  member( P, PRDS ),
  valid_p_c( P, configuration( S, R ) ).
