﻿% -----------------------------------------------------------------------------
% File: why_not_valid_product.pl
% Content: Valid product relation of FLAME (refactored)
% Author: Amador Durán Toro
% Version: 1.0
% Date: 22/07/2014
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% why_not_valid( Product, spl( Features, Model ) )
% It explains why a product is not valid.
% -----------------------------------------------------------------------------

% The product is an empty set
why_not_valid( [], _ ) :-
   !, write( 'The product contains no features.' ), nl.

% The product is not a nonempty set, i.e. contains duplicated features
why_not_valid( P, _ ) :-
   not( is_set( P ) ), !,
   write( 'Product ' ),
   writeq( P ),
   write( ' contains duplicated features.' ), nl.

% The product is not a subset of the feature set, i.e. contains unknonw features
why_not_valid( P, spl( F, _ ) ) :-
   not( subset( P, F ) ), !,
   write( 'Product ' ),
   writeq( P ),
   write( ' contains unknown features.' ), nl.

% The product is not an instance of the model
why_not_valid( P, spl( _, M ) ) :-
   not( instance_of( P, M ) ), !,
   write( 'Product ' ),
   writeq( P ),
   write( ' is not an instance of the model.' ), nl.

% The product is valid
why_not_valid( P, _ ) :-
  write( 'Product ' ),
  writeq( P ),
  write( ' is valid!' ), nl.
