﻿% -----------------------------------------------------------------------------
% File: helpers.pl
% Content: BFM helper functions for FLAME
% Author: Amador Durán Toro
% Version: 2.0
% Date: 19/03/2015 (Día del Padre)
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% Z definitions
%   children : Relationship -> F1 FeatureTree
%   t_feature_bag : FeatureTree -> bag Feature
%   r_feature_bag : Relationship -> bag Feature
%   x_feature_bag : CTC -> bag Feature
%
% Prolog predicates
%   children( Relationship, [ FeatureTree ] )
%   t_feature_bag( FeatureTree,  [ Feature ] )
%   r_feature_bag( Relationship, [ Feature ] )
%   x_feature_bag( CTC,          [ Feature ] )
% -----------------------------------------------------------------------------

% -----------------------------------------------------------------------------
% children( Relationship, [ FeatureTree ] )
% It returns the children FeatureTrees of a Relationship.
% -----------------------------------------------------------------------------

children( mandatory( Ti ), CHLD ) :-
  children( optional( Ti ), CHLD ).

children( optional( Ti ), [ Ti ] ).

children( one_or_more( T ), CHLD ) :-
  children( only_one( T ), CHLD ).

children( only_one( T ), T ).

% -----------------------------------------------------------------------------
% t_feature_bag( FeatureTree, [ Feature ] )
% It computes the bag of features in a FeatureTree.
% -----------------------------------------------------------------------------

t_feature_bag( feature( Fi, R ), B ) :-
  aggregate_all( bag( RBi ), ( member( Ri, R ), r_feature_bag( Ri, RBi ) ), RB ),
  flatten( RB, FLAT_RB ),
  append( [ Fi ], FLAT_RB, B ).

% -----------------------------------------------------------------------------
% r_feature_bag( Relationship, [ Feature ] )
% It computes the bag of features in a Relationship.
% -----------------------------------------------------------------------------

r_feature_bag( R, B ) :-
  children( R, CHLD ),
  aggregate_all( bag( TBi ), ( member( Ti, CHLD ), t_feature_bag( Ti, TBi ) ), TB ),
  flatten( TB, B ).

% -----------------------------------------------------------------------------
% x_feature_bag( CTC, [ Feature ] )
% It computes the bag of features in a CTC.
% -----------------------------------------------------------------------------

x_feature_bag( requires( F1, F2 ), F ) :-
  x_feature_bag( excludes( F1, F2 ), F ).

x_feature_bag( excludes( F1, F2 ), [ F1, F2 ] ).
